import rikai
from typing import Any, Optional
from sklearn.base import RegressorMixin, ClassifierMixin, TransformerMixin, ClusterMixin


def _get_model_type(model):
    if isinstance(model, RegressorMixin):
        return "rikai_sklearn.models.regressor"
    elif isinstance(model, ClassifierMixin):
        return "rikai_sklearn.models.classifier"
    elif isinstance(model, ClusterMixin):
        if 'predict' in dir(model):
            return "rikai_sklearn.models.cluster"
        else:
            raise RuntimeError(f"Clustering without predict method is not supported")
    elif isinstance(model, TransformerMixin):
        return "rikai_sklearn.models.transformer"
    else:
        raise RuntimeError(f"No corresponding ModelType yet")


def log_model(
    model: Any,
    registered_model_name: Optional[str] = None,
    schema: Optional[str] = None,
    customized_flavor: Optional[str] = None,
    labels: Optional[dict] = None,
    artifact_path: str = "model",
    **kwargs,
):
    model_type = _get_model_type(model)
    rikai.mlflow.sklearn.log_model(
        model,
        artifact_path,
        schema,
        registered_model_name,
        customized_flavor,
        model_type,
        labels,
        **kwargs
    )



from typing import List
from rikai_sklearn.models import SklearnModelType


class Regressor(SklearnModelType):
    def schema(self) -> str:
        return "float"

    def predict(self, x, *args, **kwargs) -> List[float]:
        assert self.model is not None
        return self.model.predict(x).tolist()

MODEL_TYPE = Regressor()




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: rikai_sklearn:rikai-sklearn

from setuptools import setup

setup(**{
    'install_requires': (
        'rikai==0.1.14',
        'scikit-learn',
    ),
    'license': 'Apache License, Version 2.0',
    'name': 'rikai-sklearn',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'rikai_sklearn',
        'rikai_sklearn.models',
    ),
    'python_requires': '>=3.7',
    'version': '0.1.1',
})



from typing import List
from rikai_sklearn.models import SklearnModelType


class Clustering(SklearnModelType):
    """Classification model type"""

    def schema(self) -> str:
        return "int"

    def predict(self, x, *args, **kwargs) -> List[int]:
        assert self.model is not None
        return self.model.predict(x).tolist()

MODEL_TYPE=Clustering()



from __future__ import annotations

from abc import ABC
from typing import Callable, List

from rikai.spark.sql.model import ModelSpec, ModelType


class SklearnModelType(ModelType, ABC):
    """Base :py:class:`ModelType` for Sklearn"""

    def __init__(self):
        self.model = None

    def load_model(self, spec: ModelSpec, **kwargs):
        self.model = spec.load_model()

    def transform(self) -> Callable:
        # Do nothing in Sklearn
        pass



from typing import List
from rikai_sklearn.models import SklearnModelType

class DimensionalityReduction(SklearnModelType):
    """Dimensionality reduction models.

    - PCA
    """

    def schema(self) -> str:
        return "array<float>"

    def predict(self, x, *args, **kwargs) -> List[float]:
        return self.model.transform(x).tolist()

MODEL_TYPE=DimensionalityReduction()
